package geniusweb.protocol.session;

import java.util.List;

import com.fasterxml.jackson.annotation.JsonSubTypes;

import geniusweb.actions.Action;
import geniusweb.issuevalue.Bid;
import geniusweb.progress.Progress;
import geniusweb.protocol.NegoState;
import geniusweb.protocol.session.saop.SAOPState;

/**
 * The current state of the session. E.g. typically contains the actions so far
 * and the parties currently connected. <br>
 * The state checks if transitions (Actions from the party) are following the
 * protocol, and thus implement most of the protocol . See
 * {@link #with(geniusweb.actions.PartyId, Action)}. <br>
 * 
 * Use {@link #with(geniusweb.actions.PartyId, Action)} and
 * {@link #with(geniusweb.actions.PartyId, tudelft.utilities.connection.Connection)}
 * to create a new state <br>
 * 
 * 
 * If protocol errors occur, these should be stored in the state and the state
 * should become {@link #isFinal()}. Throwing should happen only in case of a
 * bug.<br>
 * 
 * Implementations should be immutable (to ensure thread safety, testability
 * etc).
 * 
 * States must be serializable so that listeners can follow what is going on in
 * the protocol. As uaual, mark non-serializable fields as transient.
 */
@JsonSubTypes({ @JsonSubTypes.Type(value = SAOPState.class) })

public interface SessionState extends NegoState {

	@Override
	SessionSettings getSettings();

	/**
	 * 
	 * @return unmodifyable list of actions done so far, in the order in which
	 *         they arrived.
	 */
	List<Action> getActions();

	/**
	 * 
	 * @return the progress of the session. Can return null if the session did
	 *         not even start yet. Notice that the protocol determines when a
	 *         session officially starts (eg, in SAOP it starts after all
	 *         parties were connected succesfully).
	 */
	Progress getProgress();

	/**
	 * 
	 * @return the current standing agreement, or null if there is no agreement
	 *         at this point. An agreement does not necessarily mean
	 *         {@link #isFinal()}.
	 */
	Bid getAgreement();
}
